#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <signal.h>
#include <semaphore.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "tmodule.h"
#include "hpnsPlatform.h"
#include "hpnsConfig.h"
#ifdef __ANDROIDOS__
#include<jni.h>
#include <android/log.h>
extern jclass g_classAdp;
extern JavaVM* g_JavaVm;
#endif




int HPNS_ENG_MSG_TIMER = 0xFFF0;



void *hpnsCreateRecursiveLock()
{
    pthread_mutex_t* multex = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));

    pthread_mutexattr_t attr;

    if (pthread_mutexattr_init(&attr) != 0) {
        nprintf("fatal error that init recursive lock attr error");
        return NULL ;
    }

    if (pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE_NP) != 0) {
        nprintf("fatal error that set recursive lock type error");
        return NULL ;
    }

    if (pthread_mutex_init(multex, &attr) != 0) {
        nprintf("fatal error that init recursive lock error");
        return NULL ;
    }

    pthread_mutexattr_destroy(&attr);

    return (void*)multex ;
}

int   hpnsCloseRecursiveLock(void *lock)
{
    if (!lock) {
        nprintf("fatal error in hpnsCloseRecursiveLock that lock is null") ;
        return -1 ;
    }
    
    pthread_mutex_t* multex = (pthread_mutex_t*)lock ;

    pthread_mutex_destroy(multex);

    free(multex) ;

    return 0 ;
}

int hpnsTakeRecursiveLock(void *lock)
{
    if (!lock) {
        nprintf("fatal error in hpnsTakeRecursiveLock that lock is null") ;
        return -1 ;
    }

    pthread_mutex_t* multex = (pthread_mutex_t*)lock ;

    if (pthread_mutex_lock(multex) != 0)
        nprintf("error to take the recursive lock") ;
}

int hpnsGiveRecursiveLock(void *lock) 
{
    if (!lock) {
        nprintf("fatal error in hpnsGiveRecursiveLock that lock is null") ;
        return -1 ;
    }
    
    pthread_mutex_t* multex = (pthread_mutex_t*)lock ;

    if (pthread_mutex_unlock(multex) != 0)
        nprintf("error to give the recursive lock") ;

    return 0;
}

void   *hpnsMallocL(UINT32 size)
{
    if(0 == size) {
        nprintf("fatal error that malloc size is 0") ;
        return NULL;
    }
    
    return malloc(size);
}

void    hpnsFreeL(void* pMem)
{
    if (pMem)
        free (pMem);
}

HFILE hpnsFsOpen(char *name, UINT32 flag)
{
	int hFile = 0;

	if ( flag == HPNS_FS_CREATE )
		hFile = open(name, O_RDWR | O_CREAT, S_IRWXU);
	else if ( flag == HPNS_FS_CREATE_ALWAYS )
		hFile = open(name, O_RDWR | O_CREAT, S_IRWXU );
	else if ( flag == HPNS_FS_READ )
		hFile = open(name, O_RDONLY);
	else if ( flag == HPNS_FS_READ_WRITE)
		hFile = open(name, O_RDWR);

	if ( hFile < 0 )
	{
		nprintf("failed to open file:%s, reason:%s\n", name, strerror(errno));
		hFile = 0;
	}
	
	return hFile;
}


int hpnsFsRead(HFILE hFile, void *pData, int nLen, int *bytesread)
{
	*bytesread = read(hFile, pData, nLen);
	
	return *bytesread;
}

int hpnsFsWrite(HFILE hFile, void *pData, int nLen, int *written)
{
	int bytes;
	
	bytes = write(hFile, pData, nLen);

	if( written )
		*written = bytes;

	return bytes;
}

int hpnsFsSeek(HFILE hFile, int offset, int Whence)
{
	int flag = 0;
	
	if ( Whence == HPNS_FS_FILE_BEGIN ) flag = SEEK_SET;
	if ( Whence == HPNS_FS_FILE_CURRENT ) flag = SEEK_CUR;
	if ( Whence == HPNS_FS_FILE_END ) flag = SEEK_END;
	
    return lseek(hFile, offset, flag);
}

void hpnsFsClose(HFILE hFile)
{
       fsync(hFile) ;
	close (hFile);	
}

int hpnsFsDelete(char *file)
{
	return remove(file);
}

int hpnsFsFlush(HFILE hFile)
{
       fsync(hFile) ;
	return 0;
}

int hpnsFsTrunCate(HFILE hFile, int off_set)
{
	ftruncate(hFile,off_set);
	return 0;
}

int hpnsFsGetFileSizeWithName(char *pName, int *size)
{
	struct stat st;
	
	stat(pName, &st);
	*size = st.st_size;

	return *size;
}

int hpnsFsFileExists (char *pName)
{
	struct stat st;
	int ret;

	ret = stat(pName, &st);

	if ( ret == 0 ) 
		ret = 1;
	else
		ret = 0;

	return ret;	
}

int hpnsSendMsgToEngineP(int mid, UINT8 *pMsg, int msgLen) 
{ 
	taosSendMsgToModule( &(moduleObj[2]), 0, mid, msgLen, pMsg); 
}

int hpnsSendMsgToUIP(int mid, UINT8 *pMsg, int msgLen) 
{
  	taosSendMsgToModule( &(moduleObj[0]), 0, mid, msgLen, pMsg); 
}

void hpnsEngineProcessMsgQueue(msg_t *param)
{
	msg_t *pMsg;
	
	pMsg = (msg_t *)param;

#ifdef __ANDROIDOS__
        static jmethodID fget = 0, frelease = 0;

        JNIEnv* env = NULL;
        if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK || NULL == env) {
                nprintf("get jni env error at function: nmsGetWeakupLock");
        	return;
        }

        if(0 == fget || 0 == frelease){
                fget = (*env)->GetStaticMethodID(env, g_classAdp, "nmsGetWeakupLock","(I)V");
                frelease = (*env)->GetStaticMethodID(env, g_classAdp, "nmsReleaseWeakupLock","(I)V");
        }

#endif

	if( HPNS_ENG_MSG_TIMER == pMsg->mid )
	{
		nprintf("engine recv timer msg, timer id: %d", (UINT32)pMsg->msg);
		hpnsProcessTimer((UINT32)pMsg->msg);
		goto end;
	}

       (*env)->CallStaticVoidMethod(env,g_classAdp,fget, pMsg->mid);
	hpnsHandleMsg( pMsg->mid, pMsg->msg, pMsg->tid);
end:

#ifdef __ANDROIDOS__
	(*env)->CallStaticVoidMethod(env,g_classAdp,frelease, pMsg->mid);
	//nprintf("nmsReleaseWeakupLock");
#endif
	if ( pMsg->tid == 0 ) 
		pMsg->msg = NULL;

}

int hpnsGetAPNType()
{	
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("get jni env error at function: hpnsGetAPNType");
		return;
	}
	jmethodID midAPNType = (*env)->GetStaticMethodID(env, g_classAdp, "getAPNType","()I");
	jint IntAPNType = (*env)->CallStaticIntMethod(env,g_classAdp,midAPNType, NULL);
	nprintf("hpnsGetAPNType IntAPNType: %d ",IntAPNType);
	return IntAPNType;
#endif
}

int hpnsGetAPName(INT8 APName[])
{
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("get jni env error at function: hpnsGetAPName");
		return;
	}

	jmethodID midAPName = (*env)->GetStaticMethodID(env, g_classAdp, "getAPName","()Ljava/lang/String;");
	jstring StringAPName = (*env)->CallStaticObjectMethod(env,g_classAdp,midAPName, NULL);
	const char * APName_ret = (*env)->GetStringUTFChars(env, StringAPName, NULL);
	if(NULL == APName_ret)
	{
		nprintf("APName is null when getting APName from java");
		(*env)->ReleaseStringUTFChars(env, StringAPName, APName_ret);
		(*env)->DeleteLocalRef(env, StringAPName);
		return -1;
	}else{
		nprintf("get APName via jni: %s", APName_ret);
		if(strlen(APName_ret) >= HPNS_AP_NAME_LEN){
			strncpy(APName, APName_ret, HPNS_AP_NAME_LEN - 1);
		}else{
			strcpy(APName, APName_ret);
		}
		(*env)->ReleaseStringUTFChars(env, StringAPName, APName_ret);
		(*env)->DeleteLocalRef(env, StringAPName);
		return 0;
	}
#endif
}

void hpnsGetImsiImei(UINT8 imsi[][20], UINT8 *pImei)
{
	//strcpy(imsi[0], "460000690224717");
	//strcpy(pImei, "1234567890321");

#ifdef __ANDROIDOS__
	#define NMS_IMSI_LEN 		20
	#define NMS_IMEI_LEN            20

       	JNIEnv* env = NULL;
        if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) {
                nprintf("get jni env error at function: nmsGetPNRegistrationID");
        	return;
    	}


        jmethodID midImsi = (*env)->GetStaticMethodID(env, g_classAdp, "getIMSI","()Ljava/lang/String;");
        jstring StringArgImsi = (*env)->CallStaticObjectMethod(env,g_classAdp,midImsi, NULL);
        const char * imsi_ret = (*env)->GetStringUTFChars(env, StringArgImsi, NULL);
        if(NULL != StringArgImsi){
	if(NULL == imsi_ret){
		nprintf("imsi is null when getting imsi from java");
	}else{
		nprintf("get imsi via jni: %s", imsi_ret);
		if(strlen(imsi_ret) >= NMS_IMSI_LEN){
			strncpy(imsi[0], imsi_ret, NMS_IMSI_LEN - 1);
		}else{
			strcpy(imsi[0], imsi_ret);
		}
	}
}

        jmethodID midImei = (*env)->GetStaticMethodID(env, g_classAdp, "getIMEI","()Ljava/lang/String;");
        jstring StringArgImei = (*env)->CallStaticObjectMethod(env,g_classAdp,midImei, NULL);
        const char * imei_ret = (*env)->GetStringUTFChars(env, StringArgImei, NULL);
 if(NULL != StringArgImei){
	if(NULL == imei_ret){
		nprintf("imei is null when getting imei from java");
	}else{
		nprintf("get imei via jni: %s", imei_ret);
		if(strlen(imei_ret) >= NMS_IMEI_LEN){
			strncpy(pImei, imei_ret, NMS_IMEI_LEN - 1);
		}else{
			strcpy(pImei, imei_ret);
		}
	}
}


       (*env)->ReleaseStringUTFChars(env, StringArgImsi, imsi_ret);
       (*env)->DeleteLocalRef(env, StringArgImsi);

       (*env)->ReleaseStringUTFChars(env, StringArgImei, imei_ret);
       (*env)->DeleteLocalRef(env, StringArgImei);


	
#endif
	return;
}

UINT32 hpnsGetMobileLanguage(void)
{
#ifdef __ANDROIDOS__

	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env ");
		return;
	}
	jmethodID midLang = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetMobileLanguage", "()I");
	jint IntLang = (*env)->CallStaticIntMethod(env, g_classAdp, midLang, NULL);
	nprintf("get the mobile language:%d", IntLang);
	return IntLang;

#endif	

	return 0;
}	

char *hpnsGetTimeStamp(void) {
  static char     tstamp[40];

  struct tm      *ptm;
  struct timeval timeSecs;
  
  gettimeofday ( &timeSecs, 0);
  ptm = localtime ( &timeSecs.tv_sec );
  timeSecs.tv_usec = timeSecs.tv_usec / 1000;
  sprintf(tstamp, "%02d-%02d %02d:%02d:%02d:%03d ", (ptm->tm_mon + 1),ptm->tm_mday, ptm->tm_hour, ptm->tm_min, ptm->tm_sec, (int)timeSecs.tv_usec); 
  
  return tstamp;
}  

const char *hpnsGetThreadId(void) 
{
       static char tid[10 + 1] = {0};

    	sprintf(tid, "%d", gettid());
	return tid;
}

UINT32 hpnsGetSystemTime( )
{
	struct timeval systemTime;

	gettimeofday(&systemTime, 0);

	return systemTime.tv_sec;
}

UINT32 hpnsGetUsecTime()
{
	struct timeval systemTime;

	gettimeofday(&systemTime, 0);

	return systemTime.tv_usec;
}

UINT32 hpnsHtonl(UINT32 x)
{
	return htonl(x);
}

UINT32 hpnsNtohl(UINT32 x)
{
	return ntohl(x);
}

UINT16 hpnsHtons(UINT16 x)
{
	return htons(x);
}
	
UINT16 hpnsNtohs(UINT16 x)
{
	return ntohs(x);
}

void hpnsTrace(char* buf)
{
	fprintf(stdout, "%s", buf);
#ifdef __ANDROIDOS__
        LOGI(buf);
#endif
	return ;
}

int   hpnsGetDisplayMetrics(UINT16 *hSzie, UINT16 *vSize)
{
#ifdef __ANDROIDOS__

	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env when get DisplayMetrics ");
		return;
	}
	
	jmethodID midwidth = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetWidth", "()I");
	jint Intwidth = (*env)->CallStaticIntMethod(env, g_classAdp, midwidth, NULL);
	nprintf("get the mobile width:%d", Intwidth);
	*vSize =  Intwidth;
	
	jmethodID midheight = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetHeight", "()I");
	jint Intheight = (*env)->CallStaticIntMethod(env, g_classAdp, midheight, NULL);
	nprintf("get the mobile height:%d", Intheight);
	*hSzie =  Intheight;

	return 0;
	
#endif		

	*hSzie = 280;
	*vSize = 180;

	return 0;
}

int     hpnsGetOSInfo(INT8 clientOS[])
{
	strcpy(clientOS, "Android" );
	return 0;
}

void    hpnsGetChipSet(INT8 chipSet[])
{
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env when get moible model");
		return;
	}

	jmethodID midModel = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetModel","()Ljava/lang/String;");
	jstring StringModel = (*env)->CallStaticObjectMethod(env, g_classAdp, midModel, NULL);
	const char *pModel = (*env)->GetStringUTFChars(env, StringModel, NULL);
	if(NULL == pModel)
	{
		nprintf("model is null when getting model from java");
	}
	else
	{
		nprintf("get StringModel via jni: %s", pModel);
		if(strlen(pModel) >= HPNS_CHIPSET_LEN)
		{
			strncpy(chipSet, pModel, HPNS_CHIPSET_LEN - 1);
		}
		else
		{
			strcpy(chipSet, pModel);
		}
	}
	
	(*env)->ReleaseStringUTFChars(env, StringModel, pModel);
	(*env)->DeleteLocalRef(env, StringModel);
	
	return ;
#endif
	
	strcpy(chipSet, "linux mobile");
	return;
}

int     hpnsGetMREVersionInfo(INT8 MREVersion[])
{
#ifdef __ANDROIDOS__
	return;
#endif
	strcpy(MREVersion, "mre 1.0");
	return;

}

int  hpnsGetMemoryConfig(UINT32 *sizeOfRAM, UINT32 *sizeOfROM)
{
	*sizeOfRAM = 100;
	*sizeOfROM = 200;
	return 0;
}

int     hpnsGetCapabilities(UINT32 *voiceCap, UINT32 *videoCap, UINT32 *imageCap, UINT32	*otherCap )
{
	*otherCap = 1234;
	
		*imageCap = 2345;
		*videoCap = 4567;
		*voiceCap = 6789;
	
	return 0;
}

int     hpnsGetLocationInfo(UINT8 *latitude, UINT8 *longitude)
{

	strcpy(latitude , "100.01");
	strcpy(longitude ," 200.22");
	return 0;
}

void    hpnsGetMACAddress(INT8 macAddr[])
{
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env when get moible model");
		return;
	}

	jmethodID midMac = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetMacAdress","()Ljava/lang/String;");
	jstring StringMac= (*env)->CallStaticObjectMethod(env, g_classAdp, midMac, NULL);
	const char *pMac = (*env)->GetStringUTFChars(env, StringMac, NULL);
	if(NULL == pMac)
	{
		nprintf("model is null when getting mac address from java");
	}
	else
	{
		nprintf("get mac address via jni: %s", pMac);
		if(strlen(pMac) >= HPNS_MAC_ADDRESS_LEN)
		{
			strncpy(macAddr, pMac, HPNS_MAC_ADDRESS_LEN - 1);
		}
		else
		{
			strcpy(macAddr, pMac);
		}
	}
	
	(*env)->ReleaseStringUTFChars(env, StringMac, pMac);
	(*env)->DeleteLocalRef(env, StringMac);
	
	return ;
#endif

}

void    hpnsGetChannelID(INT32 *channelId)
{
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env when get moible model");
		return;
	}

	jmethodID mChannel = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetChannelID", "()I");
	jint chId = (*env)->CallStaticIntMethod(env, g_classAdp, mChannel, NULL);
	*channelId = chId;
	
	return ;
#endif
}

void    hpnsGetServerPortFormJava(INT16 *serverPort)
{
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env when get serverPort for java");
		return;
	}

	jmethodID mServerPort = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetServerPort", "()I");
	jint serPort = (*env)->CallStaticIntMethod(env, g_classAdp, mServerPort, NULL);
	*serverPort = serPort;
	
	return ;
#endif
}

void    hpnsGetServerPortNum(INT16 *serverPortNum)
{
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env when get serverPortNum for java ");
		return;
	}

	jmethodID mNum = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetServerPortNum", "()I");
	jint SerPortNum = (*env)->CallStaticIntMethod(env, g_classAdp, mNum, NULL);
	*serverPortNum = SerPortNum;
	
	return ;
#endif
}

void    hpnsGetServerTcpPort(INT16 *serverTcpPort)
{
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env when get serverTcpPort for java");
		return;
	}

	jmethodID mTcpPort = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetServerTcpPort", "()I");
	jint SerTcpPort = (*env)->CallStaticIntMethod(env, g_classAdp, mTcpPort, NULL);
	*serverTcpPort = SerTcpPort;
	
	return ;
#endif
}

void    hpnsGetServerIp(INT32 *serIp1, INT32 *serIp2)
{
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env when get moible model");
		return;
	}

	jmethodID mipAddress = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetServerIp", "()Ljava/lang/String;");
	jstring StringIpaddr= (*env)->CallStaticObjectMethod(env, g_classAdp, mipAddress, NULL);
	const char *ipaddr= (*env)->GetStringUTFChars(env, StringIpaddr, NULL);

    if(NULL == ipaddr){
		*serIp1 = -1;
	    *serIp2 = -1;
	}
	
	if(ipaddr){
		*serIp1 = hpnsIpStrToNum(ipaddr);
		*serIp2 = *serIp1;

		nprintf("get server ip1 address:%u", *serIp1);
	}

	(*env)->ReleaseStringUTFChars(env, StringIpaddr, ipaddr);
	(*env)->DeleteLocalRef(env, StringIpaddr);
		
	return ;
#endif
}

int hpnsGetNetConnType()
{
#ifdef __ANDROIDOS__
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
	{
		nprintf("failed to get jni env when get hpnsGetNetConnType for java ");
		return;
	}

	jmethodID tpye = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetNetConnType", "()I");
	jint netConnType = (*env)->CallStaticIntMethod(env, g_classAdp, tpye, NULL);
//	*netConnType0 = netConnType;
	
	return netConnType;
#endif
}

void hpnsGetDNSHost(char *dnsName ) 
{
#ifdef __ANDROIDOS__
		JNIEnv* env = NULL;
		if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) 
		{
			nprintf("failed to get jni env when get moible model");
			return;
		}
	
		jmethodID midDns = (*env)->GetStaticMethodID(env, g_classAdp, "adpterGetDNSHost","()Ljava/lang/String;");
		jstring StringDns= (*env)->CallStaticObjectMethod(env, g_classAdp, midDns, NULL);
		const char *dns= (*env)->GetStringUTFChars(env, StringDns, NULL);
		
		//(*env)->ReleaseStringUTFChars(env, StringDns, NULL);
		(*env)->DeleteLocalRef(env, StringDns);
		if(NULL == dns){
			 strcpy(dnsName, "hpns.hissage.net") ;
			return ;
		}

		nprintf("get server dns address:%s", dns);
		strncpy(dnsName, dns, strlen(dns)>HPNS_MAX_DOMAIN_LEN?HPNS_MAX_DOMAIN_LEN:strlen(dns));
		return  ;
#endif
   
}

UINT32 hpnsGetServerIpViaDNS(char domain[])
{
    JNIEnv* env = NULL;
    if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) {
        nprintf("get jni env error at function: hpnsGetServerIpViaDNS");
        return -1;
    }

    jstring jDomain = (*env)->NewStringUTF(env, domain);
    jmethodID midDns = (*env)->GetStaticMethodID(env, g_classAdp, "getIpViaDNS","(Ljava/lang/String;)V");
    (*env)->CallStaticVoidMethod(env,g_classAdp,midDns, jDomain);
    (*env)->DeleteLocalRef(env, jDomain);
    
    return -1;
}

#ifdef __ANDROIDOS__

#include <jni.h>
extern jclass g_classAdp;
extern JavaVM* g_JavaVm;


int hpnsFsFolderExists(char* folderName)
{
	return hpnsFsFileExists(folderName);
}


int hpnsCreateFolder(char *name)
{

	if(mkdir(name,0777))
		return -1;
	else
		return 0;
}

void nmsGetSDCardPathForLog(char* drive)
{
        JNIEnv* env = NULL;
        if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) {
                //nprintf("get jni env error at function: nmsGetStorageInfo");
        return;
    	}

        jmethodID midMain = (*env)->GetStaticMethodID(env, g_classAdp, "getSDCardPath","()Ljava/lang/String;");
        jstring StringArg = (*env)->CallStaticObjectMethod(env,g_classAdp,midMain, NULL);
        if(NULL == StringArg){
                sprintf(drive, "%s/", NULL);
                goto Error;
        }
        const char * path = (*env)->GetStringUTFChars(env, StringArg, NULL);
        sprintf(drive, "%s/nqhpns", path);

	if(!hpnsFsFolderExists(drive)){
                hpnsCreateFolder(drive);
        }

	sprintf(drive, "%s/log", drive);

	if(!hpnsFsFolderExists(drive)){
                hpnsCreateFolder(drive);
        }




        (*env)->ReleaseStringUTFChars(env, StringArg, path);
        Error:
        (*env)->DeleteLocalRef(env, StringArg);
	return;

}
void nmsGetSysStoragePath(char *drive)
{
	//strcpy((char *)drive[0], "d:\\nms3.0");
	//strcpy((char *)drive[1], "d:\\nms3.0");
    //strcpy(drive,"/data/data/com.hissage.hpe/Log/");
    	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK) {
		//nprintf("get jni env error at function: nmsGetStorageInfo");
        return;
    }

	jmethodID midMain = (*env)->GetStaticMethodID(env, g_classAdp, "getLocalDataPath","()Ljava/lang/String;");
	jstring StringArg = (*env)->CallStaticObjectMethod(env,g_classAdp,midMain, NULL);
	if(NULL == StringArg){
		sprintf(drive, "%s/", NULL);
		goto Error;
	}
	const char * path = (*env)->GetStringUTFChars(env, StringArg, NULL);
	sprintf(drive, "%s/nqhpns", path);

	
	(*env)->ReleaseStringUTFChars(env, StringArg, path);
	Error:
	(*env)->DeleteLocalRef(env, StringArg);

	return;
}

void catchSystemLog(){

        JNIEnv* env = NULL;
        if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK || NULL == env) {

         nprintf("get jni env error at function: catchSystemLog");
         return ;
        }

        jmethodID fmid = (*env)->GetStaticMethodID(env, g_classAdp, "NmsExceptionLog","()V");

        (*env)->CallStaticVoidMethod(env,g_classAdp,fmid);

        return;
}

void hpnsPostNetError(int type)
{
	JNIEnv* env = NULL;
	if ((*g_JavaVm)->GetEnv(g_JavaVm, (void**)&env, JNI_VERSION_1_2) != JNI_OK || NULL == env) {
		nprintf("get jni env error at function: postNetError");
		return;
	}

	jmethodID fmid = (*env)->GetStaticMethodID(env, g_classAdp, "postNetError","(I)V");
	(*env)->CallStaticVoidMethod(env,g_classAdp,fmid, type);

	nprintf("post net error Type: %d.",type);

}


#endif




